#!/usr/bin/python
# -*- coding:utf-8 -*-
import time
#import smbus
import smbus2 as smbus
#i2c address
ADS_I2C_ADDRESS                       = 0x48

#Pointer Register
ADS_POINTER_CONVERT               = 0x00
ADS_POINTER_CONFIG                = 0x01
ADS_POINTER_LOWTHRESH             = 0x02
ADS_POINTER_HIGHTHRESH            = 0x03

#Config Register
ADS_CONFIG_OS_BUSY                  = 0x0000      #Device is currently performing a conversion
ADS_CONFIG_OS_NOBUSY                = 0x8000      #Device is not currently performing a conversion
ADS_CONFIG_OS_SINGLE_CONVERT        = 0x8000      #Start a single conversion (when in power-down state)
ADS_CONFIG_OS_NO_EFFECT             = 0x0000      #No effect
ADS_CONFIG_MUX_MUL_0_1              = 0x0000      #Input multiplexer,AINP = AIN0 and AINN = AIN1(default)
ADS_CONFIG_MUX_MUL_0_3              = 0x1000      #Input multiplexer,AINP = AIN0 and AINN = AIN3
ADS_CONFIG_MUX_MUL_1_3              = 0x2000      #Input multiplexer,AINP = AIN1 and AINN = AIN3
ADS_CONFIG_MUX_MUL_2_3              = 0x3000      #Input multiplexer,AINP = AIN2 and AINN = AIN3
ADS_CONFIG_MUX_SINGLE_0             = 0x4000      #SINGLE,AIN0
ADS_CONFIG_MUX_SINGLE_1             = 0x5000      #SINGLE,AIN1
ADS_CONFIG_MUX_SINGLE_2             = 0x6000      #SINGLE,AIN2
ADS_CONFIG_MUX_SINGLE_3             = 0x7000      #SINGLE,AIN3
ADS_CONFIG_PGA_6144                 = 0x0000      #Gain= +/- 6.144V
ADS_CONFIG_PGA_4096                 = 0x0200      #Gain= +/- 4.096V
ADS_CONFIG_PGA_2048                 = 0x0400      #Gain= +/- 2.048V(default)
ADS_CONFIG_PGA_1024                 = 0x0600      #Gain= +/- 1.024V
ADS_CONFIG_PGA_512                  = 0x0800      #Gain= +/- 0.512V
ADS_CONFIG_PGA_256                  = 0x0A00      #Gain= +/- 0.256V
ADS_CONFIG_MODE_CONTINUOUS          = 0x0000      #Device operating mode:Continuous-conversion mode
ADS_CONFIG_MODE_NOCONTINUOUS        = 0x0100      #Device operating mode：Single-shot mode or power-down state (default)
ADS_CONFIG_DR_RATE_128              = 0x0000      #Data rate=128SPS
ADS_CONFIG_DR_RATE_250              = 0x0020      #Data rate=250SPS
ADS_CONFIG_DR_RATE_490              = 0x0040      #Data rate=490SPS
ADS_CONFIG_DR_RATE_920              = 0x0060      #Data rate=920SPS
ADS_CONFIG_DR_RATE_1600             = 0x0080      #Data rate=1600SPS
ADS_CONFIG_DR_RATE_2400             = 0x00A0      #Data rate=2400SPS
ADS_CONFIG_DR_RATE_3300             = 0x00C0      #Data rate=3300SPS
ADS_CONFIG_COMP_MODE_WINDOW         = 0x0010      #Comparator mode：Window comparator
ADS_CONFIG_COMP_MODE_TRADITIONAL    = 0x0000      #Comparator mode：Traditional comparator (default)
ADS_CONFIG_COMP_POL_LOW             = 0x0000      #Comparator polarity：Active low (default)
ADS_CONFIG_COMP_POL_HIGH            = 0x0008      #Comparator polarity：Active high
ADS_CONFIG_COMP_LAT                 = 0x0004      #Latching comparator
ADS_CONFIG_COMP_NONLAT              = 0x0000      #Nonlatching comparator (default)
ADS_CONFIG_COMP_QUE_ONE             = 0x0000      #Assert after one conversion
ADS_CONFIG_COMP_QUE_TWO             = 0x0001      #Assert after two conversions
ADS_CONFIG_COMP_QUE_FOUR            = 0x0002      #Assert after four conversions
ADS_CONFIG_COMP_QUE_NON             = 0x0003      #Disable comparator and set ALERT/RDY pin to high-impedance (default)

Config_Set = 0

class ADS1015(object):
    def __init__(self,address=ADS_I2C_ADDRESS):
        self._address = address
        self._bus = smbus.SMBus(1)

    def odrConfBit(odr):
        bits = ADS_CONFIG_DR_RATE_1600
        if odr == 128:
            bits = ADS_CONFIG_DR_RATE_128
        elif odr == 250:
            bits = ADS_CONFIG_DR_RATE_250
        elif odr == 500:
            bits = ADS_CONFIG_DR_RATE_500
        elif odr == 920:
            bits = ADS_CONFIG_DR_RATE_920
        elif odr == 1600:
            bits = ADS_CONFIG_DR_RATE_1600
        elif odr == 3300:
            bits = ADS_CONFIG_DR_RATE_3300
        return bits
    def ADS1015_SINGLE_READ(self,channel, odr=1600):                    #Read single channel data
        data=0
        Config_Set =  ( ADS_CONFIG_MODE_NOCONTINUOUS        |   #mode：Single-shot mode or power-down state    (default)
                        ADS_CONFIG_PGA_4096                 |   #Gain= +/- 4.096V                              (default)
                        ADS_CONFIG_COMP_QUE_NON             |   #Disable comparator                            (default)
                        ADS_CONFIG_COMP_NONLAT              |   #Nonlatching comparator                        (default)
                        ADS_CONFIG_COMP_POL_LOW             |   #Comparator polarity：Active low               (default)
                        ADS_CONFIG_COMP_MODE_TRADITIONAL    |   #Traditional comparator                        (default)
                        ADS1015.odrConfBit(odr)             )   #Data rate
        if channel == 0:
            Config_Set |= ADS_CONFIG_MUX_SINGLE_0
        elif channel == 1:
            Config_Set |= ADS_CONFIG_MUX_SINGLE_1
        elif channel == 2:
            Config_Set |= ADS_CONFIG_MUX_SINGLE_2
        elif channel == 3:
            Config_Set |= ADS_CONFIG_MUX_SINGLE_3
        Config_Set |=ADS_CONFIG_OS_SINGLE_CONVERT

        self._write_word(ADS_POINTER_CONFIG,Config_Set)
        while not self.ADS1015_DataReady():
            time.sleep(1e-2)
        data=self._read_u16(ADS_POINTER_CONVERT)>>4
        #print('CNF Reg after read: %x (%x)' % (cnf,Config_Set,))
        return data
    def flipEnd(self, val):
        Val_H=val&0xff
        Val_L=val>>8
        val=(Val_H<<8)|Val_L
        return val
    def _read_u16_bug(self,cmd):
        LSB = self._bus.read_byte_data(self._address,cmd)
        MSB = self._bus.read_byte_data(self._address,cmd+1)
        return (LSB << 8) + MSB
    def _read_u8(self,cmd):
        return self._bus.read_byte_data(self._address,cmd)
    def _read_u16(self,cmd):
        data = self._bus.read_word_data(self._address,cmd)
        return self.flipEnd(data)
    def _write_word(self, cmd, val):
        self._bus.write_word_data(self._address,cmd,self.flipEnd(val))

    def ADS1015_SET_CHANNEL(self,channel):
        data=0
        Config_Set =  ( ADS_CONFIG_MODE_CONTINUOUS          |   #mode：Single-shot mode or power-down state    (default)
                        ADS_CONFIG_PGA_4096                 |   #Gain= +/- 4.096V                              (default)
                        ADS_CONFIG_COMP_QUE_NON             |   #Disable comparator                            (default)
                        ADS_CONFIG_COMP_NONLAT              |   #Nonlatching comparator                        (default)
                        ADS_CONFIG_COMP_POL_LOW             |   #Comparator polarity：Active low               (default)
                        ADS_CONFIG_COMP_MODE_TRADITIONAL    |   #Traditional comparator                        (default)
                        ADS_CONFIG_DR_RATE_1600             )   #Data rate=1600SPS                             (default)
        if channel == 0:
            Config_Set |= ADS_CONFIG_MUX_SINGLE_0
        elif channel == 1:
            Config_Set |= ADS_CONFIG_MUX_SINGLE_1
        elif channel == 2:
            Config_Set |= ADS_CONFIG_MUX_SINGLE_2
        elif channel == 3:
            Config_Set |= ADS_CONFIG_MUX_SINGLE_3
#        Config_Set |=ADS_CONFIG_OS_SINGLE_CONVERT
#        print('CNF Set Reg: %x' % (Config_Set,))
        self._write_word(ADS_POINTER_CONFIG,Config_Set)

    def ADS1015_RESULT(self):
        data=self._read_u16(ADS_POINTER_CONVERT)>>4
        return data

    def ADS1015_GET_CONFIG(self):
        data=self._read_u16(ADS_POINTER_CONFIG)
        return data

    def ADS1015_DataReady(self):
        cnf = self._read_u8(ADS_POINTER_CONFIG) << 8
        if (cnf & ADS_CONFIG_OS_SINGLE_CONVERT):
            return True
        return False


if __name__ == '__main__':

    print("\nADS1015 Test Program ...\r\n")
    ads1015=ADS1015()
    state=ads1015._read_u16(ADS_POINTER_CONFIG)
    if((state & 0x8000) != 0x8000):
        print("\nADS1015 Error: %x\n" % (state,))
    print("\nADS1015 Readout Register    (0): %x\n" % (ads1015._read_u16(ADS_POINTER_CONVERT),))
    print("\nADS1015 Config Register     (1):  %x\n" % (ads1015._read_u16(ADS_POINTER_CONFIG),))
    print("\nADS1015 Low Thresh Register (2):  %x\n" % (ads1015._read_u16(ADS_POINTER_LOWTHRESH),))
    print("\nADS1015 Hi Thresh Register  (3):  %x\n" % (ads1015._read_u16(ADS_POINTER_HIGHTHRESH),))
    AIN0_DATA=ads1015.ADS1015_SET_CHANNEL(3)
    while True:
        time.sleep(0.5)
        AIN0_DATA=ads1015.ADS1015_RESULT()
        AIN1_DATA=ads1015.ADS1015_GET_CONFIG()
        AIN2_DATA=0
        AIN3_DATA=0
        print('AIN0 = %x(%dmv) ,AIN1 = %x(%dmv) ,AIN2 = %x(%dmv) AIN3 = %x(%dmv)' %
              (AIN0_DATA,AIN0_DATA*2, AIN1_DATA,AIN1_DATA*2,AIN2_DATA,AIN2_DATA*2,AIN3_DATA,AIN3_DATA*2))
