#! /usr/bin/env python3
# -*- coding:utf-8 -*-

import math

from icm20649_spi.table import flatten

class DecimatorPlain:

    lastLinesMin = []
    lastLinesMax = []
    lastLinesMean = []
    decimate = 10
    ncols = 3
    nlines = 0

    def __init__(self, decimate, ncols, output, action):
        self.decimate = decimate
        if self.decimate < 1:
            print(f'Warning: decimate is zero or below: {self.decimate}, setting to 1')
            self.decimate = 1
        self.ncols = ncols
        self.reset()
        self.outputHandle = output
        self.actionHandle = action
        print('Init', self)

    def __str__(self):
        return f"Decimate: {self.decimate}/{self.ncols}"

    def reset(self):
        self.lastLinesMin = [ math.inf] * self.ncols
        self.lastLinesMax = [-math.inf] * self.ncols
        self.lastLinesMean = [0] * self.ncols

    def processValues(self, items):
        self.nlines += 1

        for (i, val) in enumerate(items):
            self.lastLinesMin[i] = min(self.lastLinesMin[i], val)
            self.lastLinesMax[i] = max(self.lastLinesMax[i], val)
            self.lastLinesMean[i] += val

        if self.nlines % self.decimate == 0:
            # self.actionHandle(self.lastLinesMin, 'min')
            # self.actionHandle(self.lastLinesMax, 'max')
            self.actionHandle(self.lastLinesMin + self.lastLinesMax, 'minmax')
            self.lastLinesMean = [ m/self.decimate for m in self.lastLinesMean ]
            oitems = flatten([ (self.lastLinesMin[i], self.lastLinesMean[i], self.lastLinesMax[i]) for i in range(self.ncols) ])
            self.outputHandle(oitems)
            self.reset()
import numpy as np

class DecimatorNP:

    lastLinesMin = []
    lastLinesMax = []
    lastLinesMean = []
    decimate = 10
    ncols = 3
    nlines = 0
    buffer = None

    def __init__(self, decimate, ncols, output, action):
        self.decimate = decimate
        if self.decimate < 1:
            print(f'Warning: decimate is zero or below: {self.decimate}, setting to 1')
            self.decimate = 1
        self.ncols = ncols

        self.outputHandle = output
        self.actionHandle = action

        self.lastLinesMin   = np.zeros((1, self.ncols))
        self.lastLinesMax   = np.zeros((1, self.ncols))
        self.lastLinesMean  = np.zeros((1, self.ncols))

        self.buffer = np.zeros((self.decimate, self.ncols))

        print('Init', self)

    def __str__(self):
        return f"Decimate: {self.decimate}/{self.ncols}"

    def processValues(self, items):

        current = self.nlines % self.decimate
        self.buffer[current,] = items

        self.nlines += 1

        if self.nlines % self.decimate == 0:

            self.lastLinesMin = np.amin(self.buffer, axis=0)
            self.lastLinesMax = np.amax(self.buffer, axis=0)
            self.lastLinesMean = np.mean(self.buffer, axis=0)

            minmaxarray = np.concatenate((self.lastLinesMin, self.lastLinesMax))
            self.actionHandle(minmaxarray, 'minmax')

            decimArray = np.hstack((self.lastLinesMin.reshape((self.ncols,1)),
                                    self.lastLinesMean.reshape((self.ncols,1)),
                                    self.lastLinesMax.reshape((self.ncols,1))))
            decimArray = decimArray.reshape((self.ncols*3,1))

            self.outputHandle(decimArray)

Decimator = DecimatorPlain
#Decimator = DecimatorNP
